#include "vtkEdgeFaceReader.h"

#include "vtkEdgeFaceDataSet.h"

#include "vtkGenericAttributeCollection.h"
#include "vtkInformation.h"
#include "vtkInformationVector.h"
#include "vtkObjectFactory.h"

#include <vtkstd/string>
#include <vtkstd/vector>
//#include <vtkstd/ifstream>

vtkStandardNewMacro(vtkEdgeFaceReader);
vtkCxxRevisionMacro(vtkEdgeFaceReader,"$Revision:$");

class vtkEdgeFaceReader::Private
{
public:
  vtkIdType NumberOfPoints;
  vtkIdType NumberOfHexes;
  vtkstd::vector<vtkstd::string> EdgeFieldNames;
  vtkstd::vector<vtkstd::string> FaceFieldNames;
};

vtkEdgeFaceReader::vtkEdgeFaceReader()
{
  this->Data = new vtkEdgeFaceReader::Private;
  this->FileName = 0;
}

vtkEdgeFaceReader::~vtkEdgeFaceReader()
{
  delete this->Data;
  this->SetFileName( 0 );
}

int vtkEdgeFaceReader::RequestDataObject(
  vtkInformation* vtkNotUsed(request), vtkInformationVector** vtkNotUsed(inInfo), vtkInformationVector* outInfo )
{
  if ( ! outInfo )
    {
    return 0;
    }
  vtkInformation* info = outInfo->GetInformationObject( 0 );
  if ( ! info )
    {
    return 0;
    }

  vtkEdgeFaceDataSet* output = vtkEdgeFaceDataSet::SafeDownCast( info->Get( vtkDataObject::DATA_OBJECT() ) );
  if ( ! output )
    {
    output = vtkEdgeFaceDataSet::New();
    output->SetPipelineInformation( info );
    output->Delete();
    this->GetOutputPortInformation( 0 )->Set( vtkDataObject::DATA_EXTENT_TYPE(), output->GetExtentType() );
    }

  return 1;
}

int vtkEdgeFaceReader::RequestInformation(
  vtkInformation* request, vtkInformationVector** inInfo, vtkInformationVector* outInfo )
{
  return 1;
}

int vtkEdgeFaceReader::RequestData(
  vtkInformation* request, vtkInformationVector** inInfo, vtkInformationVector* outInfo )
{
  return 1;
}

void vtkEdgeFaceReader::PrintSelf( ostream& os, vtkIndent indent )
{
  this->Superclass::PrintSelf( os, indent );
  if ( this->FileName )
    {
    os << "FileName: \"" << this->FileName << "\"\n";
    }
  else
    {
    os << "FileName: (null)\n";
    }
  os << "Data: " << this->Data << "\n";
}

