#include "vtkEdgeFaceAttribute.h"
#include "vtkEdgeFaceCell.h"
#include "vtkEdgeFaceCellIterator.h"
#include "vtkEdgeFaceDataSet.h"
#include "vtkEdgeFacePointIterator.h"
#include "vtkEdgeFaceReader.h"

#include "vtkGenericAttributeCollection.h"

int main( int argc, char* argv[] )
{
  const char* fname = argc > 1 ? argv[1] : "data.edgeface";

  vtkEdgeFaceReader* efr = vtkEdgeFaceReader::New();
  efr->SetFileName( fname );
  efr->Update();

  vtkEdgeFaceDataSet* ef = vtkEdgeFaceDataSet::SafeDownCast( efr->GetOutput() );
  vtkGenericAttributeCollection* gac = ef->GetAttributes();
  int na = gac->GetNumberOfAttributes();
  vtkEdgeFaceAttribute* att;
  for ( int i = 0; i < na; ++ i )
    {
    att = vtkEdgeFaceAttribute::SafeDownCast( gac->GetAttribute( i ) );
    if ( ! att )
      continue;

    cout
      << "Attribute: " << att->GetName() << " "
      << "Components: " << att->GetNumberOfComponents() << " "
      << "Centering: " << att->GetCentering() << " "
      << "Storage: " << att->GetComponentType() << " "
      << "\n";
    }

  ef->ComputeNeighbors();
  double midpt[] = { 0.5, 0.5, 0.5 };
  vtkEdgeFaceCellIterator* cit = vtkEdgeFaceCellIterator::SafeDownCast( ef->NewCellIterator() );
  for ( cit->Begin(); ! cit->IsAtEnd(); cit->Next() )
    {
    cout << "Cell conn offset: " << cit->GetConnectivityOffset() << " Boundary: " << cit->GetBoundary() << "\n";
    cout << "  Value of " << att->GetName() << " on corners:\n";
    double* tvals = att->GetTuple( cit->GetCell() );
    if ( tvals )
      {
      for ( int i = 0; i < 8; ++i, tvals += 3 )
        {
        cout << "    ( " << tvals[0] << ", " << tvals[1] << ", " << tvals[2] << " )\n";
        }
      }
    cout << "  Boundary? [";
    for ( int b = 0; b < 6; ++ b )
      {
      cout << ( cit->GetCell()->IsFaceOnBoundary( b ) ? "*" : "-" );
      }
    cout << "]\n";
    cout << "  Midpt (";
    double x[3];
    cit->GetCell()->EvaluateLocation( 0, midpt, x );
    cout << x[0] << ", " << x[1] << ", " << x[2] << ")\n";
    }
  cit->Delete();

  cout << "Iterate over all points in mesh:";
  vtkEdgeFacePointIterator* pit = vtkEdgeFacePointIterator::SafeDownCast( ef->NewPointIterator() );
  for ( pit->Begin(); ! pit->IsAtEnd(); pit->Next() )
    {
    cout << " " << pit->GetId();
    }
  cout << "\n";
  pit->Delete();

  cout << "Iterate over all points in cell 1:";
  pit = vtkEdgeFacePointIterator::SafeDownCast( ef->NewPointIterator() );
  pit->SetMode( vtkEdgeFacePointIterator::CELL );
  pit->SetCellId( 1 );
  for ( pit->Begin(); ! pit->IsAtEnd(); pit->Next() )
    {
    cout << " " << pit->GetId();
    }
  cout << "\n";
  pit->Delete();

  cout << "Iterate over all points in cell 1 face 0:";
  pit = vtkEdgeFacePointIterator::SafeDownCast( ef->NewPointIterator() );
  pit->SetMode( vtkEdgeFacePointIterator::BOUNDARY );
  pit->SetCellId( 1 );
  pit->SetBoundary( 0 );
  pit->SetDimension( 2 );
  for ( pit->Begin(); ! pit->IsAtEnd(); pit->Next() )
    {
    cout << " " << pit->GetId();
    }
  cout << "\n";
  pit->Delete();

  cout << "Iterate over all points in cell 1 edge 0:";
  pit = vtkEdgeFacePointIterator::SafeDownCast( ef->NewPointIterator() );
  pit->SetMode( vtkEdgeFacePointIterator::BOUNDARY );
  pit->SetCellId( 1 );
  pit->SetBoundary( 0 );
  pit->SetDimension( 1 );
  for ( pit->Begin(); ! pit->IsAtEnd(); pit->Next() )
    {
    cout << " " << pit->GetId();
    }
  cout << "\n";
  pit->Delete();

  efr->Delete();
  return 0;
}
