// .NAME vtkEdgeFaceAttribute - Objects that manage some attribute data.
// .SECTION Description

#include "vtkEdgeFaceAttribute.h"

#include "vtkEdgeFaceDataSet.h"
#include "vtkEdgeFaceCell.h"
#include "vtkEdgeFaceCellIterator.h"
#include "vtkEdgeFacePointIterator.h"

#include "vtkDataSetAttributes.h"
#include "vtkDoubleArray.h"
#include "vtkObjectFactory.h"

#include <assert.h>

vtkStandardNewMacro(vtkEdgeFaceAttribute);
vtkCxxRevisionMacro(vtkEdgeFaceAttribute, "$Revision: 1.1 $");
vtkCxxSetObjectMacro(vtkEdgeFaceAttribute,Array,vtkDoubleArray);
vtkCxxSetObjectMacro(vtkEdgeFaceAttribute,DataSet,vtkEdgeFaceDataSet);

vtkEdgeFaceAttribute::vtkEdgeFaceAttribute()
{
  this->Array = vtkDoubleArray::New();
  this->Array->SetName( "Empty" );
  this->Array->SetNumberOfComponents( 3 );
  this->Type = vtkDataSetAttributes::VECTORS;
  this->Centering = vtkBoundaryCentered;
  this->DataSet = 0;
  this->DeRhamBasis = FACE;
}

vtkEdgeFaceAttribute::~vtkEdgeFaceAttribute()
{
  this->SetArray( 0 );
  this->SetDataSet( 0 );
}

void vtkEdgeFaceAttribute::PrintSelf( ostream& os, vtkIndent indent )
{
  this->Superclass::PrintSelf( os, indent );
  os << "DataSet: " << this->DataSet << "\n";
  os << "Array: " << this->Array << "\n";
  os << "Type: " << this->Type << "\n";
  os << "Centering: " << this->Centering << "\n";
}

const char* vtkEdgeFaceAttribute::GetName()
{
  return this->Array->GetName();
}

int vtkEdgeFaceAttribute::GetNumberOfComponents()
{
  return this->Array->GetNumberOfComponents();
}

int vtkEdgeFaceAttribute::GetCentering()
{
  return this->Centering;
}

int vtkEdgeFaceAttribute::GetType()
{
  return this->Type;
}

int vtkEdgeFaceAttribute::GetComponentType()
{
  return VTK_DOUBLE;
}

vtkIdType vtkEdgeFaceAttribute::GetSize()
{
  return this->Array->GetNumberOfTuples();
}

unsigned long vtkEdgeFaceAttribute::GetActualMemorySize()
{
  return this->Array->GetActualMemorySize();
}

double* vtkEdgeFaceAttribute::GetRange( int component )
{
  return this->Array->GetRange( component );
}

void vtkEdgeFaceAttribute::GetRange( int component, double range[2] )
{
  return this->Array->GetRange( range, component );
}

double vtkEdgeFaceAttribute::GetMaxNorm()
{
  return this->Array->GetMaxNorm();
}

double* vtkEdgeFaceAttribute::GetTuple( vtkGenericAdaptorCell* c )
{
  vtkEdgeFaceCell* cell = vtkEdgeFaceCell::SafeDownCast( c );
  if ( ! cell )
    return 0;

  return cell->GetAttributeAtCorners( this );
}

void vtkEdgeFaceAttribute::GetTuple( vtkGenericAdaptorCell* c, double* tuple )
{
  vtkEdgeFaceCell* cell = vtkEdgeFaceCell::SafeDownCast( c );
  if ( ! cell )
    return;

  double* vals = cell->GetAttributeAtCorners( this );
  if ( ! vals )
    {
    return;
    }
  for ( int i = 0; i < 8; ++ i )
    {
    tuple[i] = vals[i];
    }
}

double* vtkEdgeFaceAttribute::GetTuple( vtkGenericCellIterator* c )
{
  vtkEdgeFaceCell* cell = vtkEdgeFaceCell::SafeDownCast( c ? c->GetCell() : 0 );
  if ( ! cell )
    return 0;

  return cell->GetAttributeAtCorners( this );
}

void vtkEdgeFaceAttribute::GetTuple( vtkGenericCellIterator* c, double* tuple )
{
  double* vals = this->GetTuple( c );
  if ( ! vals )
    {
    return;
    }
  for ( int i = 0; i < 8; ++ i )
    {
    tuple[i] = vals[i];
    }
}

double* vtkEdgeFaceAttribute::GetTuple( vtkGenericPointIterator* p )
{
  (void) p;
  // FIXME
  return 0;
}

void vtkEdgeFaceAttribute::GetTuple( vtkGenericPointIterator* p, double* tuple )
{
  (void) p;
  (void) tuple;
  // FIXME
}

void vtkEdgeFaceAttribute::GetComponent( int i, vtkGenericCellIterator* c, double* values )
{
  (void) i;
  (void) c;
  (void) values;
  // FIXME
}

double vtkEdgeFaceAttribute::GetComponent( int i, vtkGenericPointIterator* p )
{
  (void) i;
  (void) p;
  // FIXME
  return 0.;
}

void vtkEdgeFaceAttribute::DeepCopy( vtkGenericAttribute* other )
{
  vtkEdgeFaceAttribute* att = vtkEdgeFaceAttribute::SafeDownCast( other );
  assert( att );

  this->Array->DeepCopy( att->GetArray() );
  this->SetDataSet( att->GetDataSet() );
  this->Centering = other->GetCentering();
  this->Type = other->GetType();
}

void vtkEdgeFaceAttribute::ShallowCopy( vtkGenericAttribute* other )
{
  vtkEdgeFaceAttribute* att = vtkEdgeFaceAttribute::SafeDownCast( other );
  assert( att );

  this->SetArray( att->GetArray() );
  this->SetDataSet( att->GetDataSet() );
  this->Centering = other->GetCentering();
  this->Type = other->GetType();
}


