#include "vtkEdgeFaceCellIterator.h"

#include "vtkEdgeFaceDataSet.h"
#include "vtkEdgeFaceCell.h"

#include "vtkCellArray.h"
#include "vtkGenericAdaptorCell.h"
#include "vtkObjectFactory.h"

vtkStandardNewMacro(vtkEdgeFaceCellIterator);
vtkCxxRevisionMacro(vtkEdgeFaceCellIterator, "$Revision: 1.1 $");
vtkCxxSetObjectMacro(vtkEdgeFaceCellIterator,DataSet,vtkEdgeFaceDataSet);
vtkCxxSetObjectMacro(vtkEdgeFaceCellIterator,Cell,vtkEdgeFaceCell);

const int vtkEdgeFaceCellIterator::BoundariesOfDimension[] =
{
  8,
  12,
  6,
  1,
  8 + 12 + 6
};

vtkEdgeFaceCellIterator::vtkEdgeFaceCellIterator()
{
  this->DataSet = 0;
  this->Mode = MESH;
  this->Dimension = -1;
  this->Cell = 0; // Don't create one until we need one.
}

vtkEdgeFaceCellIterator::~vtkEdgeFaceCellIterator()
{
  this->SetDataSet( 0 ); // release reference to dataset
  this->SetCell( 0 );
}

void vtkEdgeFaceCellIterator::PrintSelf( ostream& os, vtkIndent indent )
{
  this->Superclass::PrintSelf(os,indent);
  os << indent << "DataSet: " << this->DataSet << "\n";
  os << indent << "Mode: ";
  switch ( this->Mode )
    {
  case MESH:
    os << "Mesh";
    break;
  case COFACETS:
    os << "Facets";
    break;
  case BOUNDARY:
    os << "Boundary facets";
    break;
  default:
    os << "(invalid)";
    break;
    }
  os << "\n";
  os << indent << "Dimension: " << this->Dimension << "\n";
  os << indent << "Cell: " << this->Cell << "\n";
  os << indent << "Boundary: " << this->Boundary << "\n";
}

void vtkEdgeFaceCellIterator::Begin()
{
  this->Boundary = 0;
  this->ConnectivityOffset = 0;
  // FIXME: Handle BOUNDARY and COFACETS mode.
  if ( ! this->Cell )
    {
    this->Cell = vtkEdgeFaceCell::SafeDownCast( this->NewCell() );
    }
  this->Cell->SetId( 0 );
}

int vtkEdgeFaceCellIterator::IsAtEnd()
{
  return this->ConnectivityOffset < 0;
}

vtkGenericAdaptorCell* vtkEdgeFaceCellIterator::NewCell()
{
  vtkEdgeFaceCell* cell = vtkEdgeFaceCell::New();
  cell->SetDataSet( this->DataSet );
  return cell;
}

void vtkEdgeFaceCellIterator::GetCell( vtkGenericAdaptorCell* c )
{
  if ( ! c )
    {
    return;
    }
  // FIXME
}

vtkGenericAdaptorCell* vtkEdgeFaceCellIterator::GetCell()
{
  if ( this->ConnectivityOffset < 0 )
    { // We are at the end of the iteration, or haven't started. There is no cell.
    return 0;
    }

  if ( ! this->Cell )
    {
    vtkEdgeFaceCell* cell = vtkEdgeFaceCell::SafeDownCast( this->NewCell() );
    this->SetCell( cell );
    cell->Delete(); // release the extra reference
    }

  // FIXME: Use this->ConnectivityOffset...
  return this->Cell;
}

void vtkEdgeFaceCellIterator::Next()
{
  IterationMode m = (IterationMode) this->Mode;
  switch ( m )
    {
  case MESH:
    this->ConnectivityOffset += 8 + 12 + 6 + 1; // + 1 for the number of entries/cell
    break;
  case BOUNDARY:
      {
      int didx = this->Dimension < 0 ? 4 : this->Dimension;
      do
        {
        if ( ++ this->Boundary >= vtkEdgeFaceCellIterator::BoundariesOfDimension[didx] )
          {
          this->ConnectivityOffset += 8 + 12 + 6 + 1; // + 1 for the number of entries/cell
          this->Boundary = 0;
          }
        } while ( ! 1 ); // this->IsOnBoundary() );
      }
    break;
  case COFACETS:
    // FIXME
    break;
    }
  // See if we have stepped past end. If so, set state to avoid mistakes.
  if ( this->ConnectivityOffset >= this->DataSet->GetConnectivity()->GetNumberOfConnectivityEntries() )
    {
    this->ConnectivityOffset = -1;
    }
  if ( this->Cell )
    this->Cell->SetId( this->ConnectivityOffset / 27 );
}

