#include "vtkEdgeFaceFieldSampler.h"

#include "vtkEdgeFaceAttribute.h"
#include "vtkEdgeFaceCell.h"
#include "vtkEdgeFaceDataSet.h"

#include "vtkCellData.h"
#include "vtkDoubleArray.h"
#include "vtkGenericAttributeCollection.h"
#include "vtkGenericCellIterator.h"
#include "vtkInformation.h"
#include "vtkInformationVector.h"
#include "vtkObjectFactory.h"
#include "vtkPointData.h"
#include "vtkPolyData.h"

#include <vtkstd/vector>

vtkStandardNewMacro(vtkEdgeFaceFieldSampler);
vtkCxxRevisionMacro(vtkEdgeFaceFieldSampler,"$Revision:$");

vtkEdgeFaceFieldSampler::vtkEdgeFaceFieldSampler()
{
  this->SamplesPerCell = 100;
}

vtkEdgeFaceFieldSampler::~vtkEdgeFaceFieldSampler()
{
}

void vtkEdgeFaceFieldSampler::PrintSelf( ostream& os, vtkIndent indent )
{
  this->Superclass::PrintSelf( os, indent );
  os << indent << "SamplesPerCell: " << this->SamplesPerCell << "\n";
}

int vtkEdgeFaceFieldSampler::FillInputPortInformation( int vtkNotUsed(port), vtkInformation* info )
{
  info->Set( vtkDataObject::DATA_TYPE_NAME(), "vtkGenericDataSet" );
  return 1;
}

int vtkEdgeFaceFieldSampler::RequestData(
  vtkInformation* vtkNotUsed(req), vtkInformationVector** inpInfo, vtkInformationVector* outInfo )
{
  if ( ! inpInfo )
    {
    return 0;
    }
  vtkInformation* inInfo = inpInfo[0]->GetInformationObject( 0 );
  if ( ! inInfo )
    {
    return 0;
    }
  vtkEdgeFaceDataSet* input = vtkEdgeFaceDataSet::SafeDownCast( inInfo->Get( vtkDataObject::DATA_OBJECT() ) );
  if ( ! input )
    {
    return 0;
    }

  if ( ! outInfo )
    {
    return 0;
    }
  vtkInformation* ouInfo = outInfo->GetInformationObject( 0 );
  if ( ! ouInfo )
    {
    return 0;
    }
  vtkPolyData* output = vtkPolyData::SafeDownCast( ouInfo->Get( vtkDataObject::DATA_OBJECT() ) );
  if ( ! output )
    {
    return 0;
    }

  return this->SampleFields( input, input->GetAttributes(), output );
}

void vtkEdgeFaceFieldSamplerAddSample(
  vtkGenericAdaptorCell* cell, double pcoords[3], vtkGenericAttributeCollection* atts,
  vtkPolyData* pd, vtkIdType& pt,
  vtkstd::vector<vtkDoubleArray*>& attPtrs, vtkstd::vector<int>& attOffs, vtkstd::vector<double>& attVals )
{
  double x[3];
  cell->EvaluateLocation( 0 /*subid*/, pcoords, x );
  pd->GetPoints()->SetPoint( ++ pt, x );
  pd->InsertNextCell( VTK_VERTEX, 1, &pt );
  cell->InterpolateTuple( atts, pcoords, &attVals[0] );
  int na = static_cast<int>( attPtrs.size() );
  for ( int i = 0; i < na; ++ i )
    {
    attPtrs[i]->InsertNextTuple( &attVals[attOffs[i]] );
    }
}

int vtkEdgeFaceFieldSampler::SampleFields( vtkEdgeFaceDataSet* ef, vtkGenericAttributeCollection* atts, vtkPolyData* pd )
{
  vtkEdgeFaceAttribute* att;
  vtkPoints* outpts = vtkPoints::New();
  vtkIdType np = ef->GetNumberOfCells();
  outpts->SetNumberOfPoints( this->SamplesPerCell * np );
  pd->Allocate();
  pd->SetPoints( outpts );
  outpts->Delete();

  vtkIdType pt = -1;
  double pcoords[3]; // = { 0.5, 0.5, 0.5 };
  int off = 0;
  vtkstd::vector<double> attVals;
  vtkstd::vector<int> attOffs;
  vtkstd::vector<vtkDoubleArray*> attPtrs;
  attOffs.push_back( off );
  int na = atts->GetNumberOfAttributes();
  for ( int i = 0; i < na; ++ i )
    {
    att = vtkEdgeFaceAttribute::SafeDownCast( atts->GetAttribute( i ) ); 
    if ( ! att )
      {
      continue;
      }
    int nc = att->GetNumberOfComponents();
    off += nc;
    attOffs.push_back( off );
    vtkDoubleArray* arr = vtkDoubleArray::New();
    arr->SetName( att->GetName() );
    arr->SetNumberOfComponents( nc );
    pd->GetPointData()->AddArray( arr );
    arr->Delete();
    attPtrs.push_back( arr );
    }
  attVals.resize( off );
  vtkGenericCellIterator* cit = ef->NewCellIterator();
  for ( cit->Begin(); ! cit->IsAtEnd(); cit->Next() )
    {
    for ( int i = 0; i < this->SamplesPerCell; ++ i )
      {
      for ( int j = 0; j < 3; ++ j )
        {
        pcoords[j] = drand48();
        }
      vtkEdgeFaceFieldSamplerAddSample( cit->GetCell(), pcoords, atts, pd, pt, attPtrs, attOffs, attVals );
      }
    }
  cit->Delete();
  return 1;
}

