#ifndef __vtkEdgeFaceFieldSampler_h
#define __vtkEdgeFaceFieldSampler_h
// .NAME vtkEdgeFaceFieldSampler - Sample a vector field at points inside each cell.
// .SECTION Description
// Edge/face attributes do not have continuity across cell boundaries (actually, they
// are continuous in some directions but not others), so sampling the vector field
// interior to the element makes sense.
// The output of this filter is polydata containing a set of points with vector field
// values at those points.

#include "vtkPolyDataAlgorithm.h"

class vtkEdgeFaceDataSet;
class vtkGenericAttributeCollection;

class VTK_EXPORT vtkEdgeFaceFieldSampler : public vtkPolyDataAlgorithm
{
public:
  static vtkEdgeFaceFieldSampler* New();
  virtual void PrintSelf( ostream& os, vtkIndent indent );
  vtkTypeRevisionMacro(vtkEdgeFaceFieldSampler,vtkPolyDataAlgorithm);

  // Description:
  // How many samples should be generated per cell?
  // Random parametric coordinates will be chosen for each sample and
  // the corresponding world coordinates and vector fields will be output.
  vtkSetMacro(SamplesPerCell,int);
  vtkGetMacro(SamplesPerCell,int);

protected:
  vtkEdgeFaceFieldSampler();
  virtual ~vtkEdgeFaceFieldSampler();

  // Description:
  // Specify the output of this filter is polydata.
  int FillInputPortInformation( int vtkNotUsed(port), vtkInformation* info );

  // Description:
  // Perform the sampling.
  virtual int RequestData( vtkInformation* request, vtkInformationVector** inInfo, vtkInformationVector* outInfo );

  // Description:
  // Really perform the sampling.
  virtual int SampleFields( vtkEdgeFaceDataSet* ef, vtkGenericAttributeCollection* atts, vtkPolyData* pd );

  int SamplesPerCell;

private:
  vtkEdgeFaceFieldSampler( const vtkEdgeFaceFieldSampler& ); // Not implemented.
  void operator = ( const vtkEdgeFaceFieldSampler& ); // Not implemented.
};

#endif // __vtkEdgeFaceFieldSampler_h
