#include "vtkEdgeFacePointIterator.h"

#include "vtkEdgeFaceCell.h"
#include "vtkEdgeFaceDataSet.h"

#include "vtkCellArray.h"
#include "vtkObjectFactory.h"

vtkStandardNewMacro(vtkEdgeFacePointIterator);
vtkCxxRevisionMacro(vtkEdgeFacePointIterator, "$Revision: 1.1 $");
vtkCxxSetObjectMacro(vtkEdgeFacePointIterator,DataSet,vtkEdgeFaceDataSet);

vtkEdgeFacePointIterator::vtkEdgeFacePointIterator()
{
  this->DataSet = 0;
  this->Id = -1;
  this->Mode = MESH;
  this->CellId = -1;
  this->Dimension = -1;
  this->Boundary = -1;
  this->Index = -1;
}

vtkEdgeFacePointIterator::~vtkEdgeFacePointIterator()
{
  this->SetDataSet( 0 ); // release reference to dataset
}

void vtkEdgeFacePointIterator::PrintSelf(ostream& os, vtkIndent indent)
{
  this->Superclass::PrintSelf(os,indent);
  os << indent << "DataSet: " << this->DataSet << "\n";
  os << indent << "Id: " << this->Id << "\n";
  os << indent << "Mode: ";
  switch ( this->Mode )
    {
  case MESH:
    os << "MESH";
    break;
  case BOUNDARY:
    os << "BOUNDARY";
    break;
  case CELL:
    os << "CELL";
    break;
  default:
    os << "(invalid: " << static_cast<int>( this->Mode ) << ")";
    }
  os << "\n";
  os << indent << "Dimension: " << this->Dimension << "\n";
  os << indent << "Boundary: " << this->Boundary << "\n";
  os << indent << "Index: " << this->Index << "\n";
}

void vtkEdgeFacePointIterator::Begin()
{
  switch ( this->Mode )
    {
  case MESH:
    this->Id = 0;
    // Set a few values to cause trouble if someone changes the Mode midway through traversal
    this->CellId = -1;
    this->Dimension = -1;
    this->Boundary = -1;
    break;
  case CELL:
  case BOUNDARY:
      {
      vtkCellArray* cellArr = this->DataSet->GetConnectivity();
      vtkIdType* conn;
      vtkIdType npts;
      cellArr->GetCell( 27 * this->CellId, npts, conn );
      if ( this->Mode == CELL )
        {
        this->Id = conn[0];
        this->Index = 0;
        }
      else
        {
        this->Index = 0;
        switch ( this->Dimension )
          {
        case 2:
          this->Id = conn[vtkEdgeFaceCell::FaceArrays[this->Boundary][this->Index]];
          break;
        case 1:
          this->Id = conn[vtkEdgeFaceCell::EdgeArrays[this->Boundary][this->Index]];
          break;
        default:
        case 0:
          this->Id = conn[this->Index];
          break;
          }
        }
      }
    }
}

int vtkEdgeFacePointIterator::IsAtEnd()
{
  switch ( this->Mode )
    {
  case MESH:
    return this->Id >= this->DataSet->GetNumberOfPoints();
  case CELL:
    // FIXME: Is the cell a hex or a quad or an edge?
    return this->Index >= 8;
  case BOUNDARY:
    return this->Id <= 0;
    }
  return 1;
}

void vtkEdgeFacePointIterator::Next()
{
  switch ( this->Mode )
    {
  case MESH:
    ++ this->Id;
    break;
  case CELL:
  case BOUNDARY:
      {
      vtkCellArray* cellArr = this->DataSet->GetConnectivity();
      vtkIdType* conn;
      vtkIdType npts;
      cellArr->GetCell( 27 * this->CellId, npts, conn );
      ++ this->Index;
      if ( this->Mode == CELL )
        {
        this->Id = conn[this->Index];
        }
      else
        {
        switch ( this->Dimension )
          {
        case 2:
          this->Id = this->Index >= 4 ? -1 : conn[vtkEdgeFaceCell::FaceArrays[this->Boundary][this->Index]];
          break;
        case 1:
          this->Id = this->Index >= 2 ? -1 : conn[vtkEdgeFaceCell::EdgeArrays[this->Boundary][this->Index]];
          break;
        default:
        case 0:
          this->Id = this->Index >= 12 ? -1 : conn[this->Index];
          break;
          }
        }
      }
    break;
    }
}

double* vtkEdgeFacePointIterator::GetPosition()
{
  return this->DataSet->GetPoint( this->Id );
}

void vtkEdgeFacePointIterator::GetPosition( double x[3] )
{
  this->DataSet->GetPoint( this->Id, x );
}

vtkIdType vtkEdgeFacePointIterator::GetId()
{
  return this->Id;
}

